-- Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY SDRAM_CTRL IS
PORT(
CLK: IN std_logic;
CLK100M: IN std_logic;
RST_N: IN std_logic;
VgaHsync_edge : in std_logic;
CamHsync_edge : in std_logic;
CamVsync : in std_logic;
CamVsync_edge : in std_logic;
		CamLineCount : in std_logic_vector(8 downto 0);
		CamPixCount4x : in std_logic_vector(15 downto 0);
		VgaLineCount : in std_logic_vector(8 downto 0);
		VgaPixCount : in std_logic_vector(9 downto 0);
  buf_RGB : in std_logic_vector(15 downto 0);
		latch0	: out std_logic_vector(15 downto 0);
		latch1	: out std_logic_vector(15 downto 0);
		latch2	: out std_logic_vector(15 downto 0);
		latch3	: out std_logic_vector(15 downto 0);
  vga_RGB_latch0 : in std_logic_vector(15 downto 0);
  vga_RGB_latch1 : in std_logic_vector(15 downto 0);
  vga_RGB_latch2 : in std_logic_vector(15 downto 0);
  vga_RGB_latch3 : in std_logic_vector(15 downto 0);
		winp0: out std_logic;
		winp1: out std_logic;
		winp2: out std_logic;
		winp3: out std_logic;
  SDRAM_DQENB_N: out std_logic;
  SDRAM_DQMH: out std_logic;
  SDRAM_DQML: out std_logic;
  SDRAM_CS_N: out std_logic;
  SDRAM_WE_N: out std_logic;
  SDRAM_RAS_N: out std_logic;
  SDRAM_CAS_N: out std_logic;
  SDRAM_CKE: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0)
 );
END SDRAM_CTRL;

ARCHITECTURE structure OF SDRAM_CTRL IS

 signal buf_RGB_dly1 : std_logic_vector(15 downto 0);
 signal buf_RGB_dly2 : std_logic_vector(15 downto 0);
 signal buf_RGB_dly3 : std_logic_vector(15 downto 0);
 signal sdram_data0, sdram_data1, sdram_data2, sdram_data3 : std_logic_vector(15 downto 0);
 signal buf_RGB_win : std_logic;
 signal MemState : std_logic_vector(4 downto 0);
 signal Initial : std_logic_vector(4 downto 0) := "00000";
 signal MRS : std_logic_vector(4 downto 0) := "00010";
 signal BstWr : std_logic_vector(4 downto 0) := "00100";
 signal BstRd : std_logic_vector(4 downto 0) := "00101";
 signal PreChAll : std_logic_vector(4 downto 0) := "00001";
 signal doingPreChAll : std_logic_vector(4 downto 0) := "10001";
 signal doingMRS : std_logic_vector(4 downto 0) := "10010";
 signal cmdcount : std_logic_vector(3 downto 0);
 signal count100m : std_logic_vector(4 downto 0);
 signal sdram_data : std_logic_vector(15 downto 0);
 signal csn, wrn, rasn, casn : std_logic;
 signal dqm, dqenbn : std_logic;
 signal cmdFound : std_logic;
 signal baddr, baddr_norm : std_logic_vector(1 downto 0);
 signal coladdr : std_logic_vector(11 downto 0);
 signal rowaddr : std_logic_vector(11 downto 0);
 signal pwrupcount : std_logic_vector(15 downto 0);
 signal pwrupready : std_logic;
 signal oddLine2x : std_logic;
 signal FrameCount, frame2write, frame2read, frame : std_logic_vector(2 downto 0);
 signal SDRAM_DQENB_N_sig: std_logic;
 signal SDRAM_DQMH_sig: std_logic;
 signal SDRAM_DQML_sig: std_logic;
 signal SDRAM_CS_N_sig: std_logic;
 signal SDRAM_WE_N_sig: std_logic;
 signal SDRAM_RAS_N_sig: std_logic;
 signal SDRAM_CAS_N_sig: std_logic;
 signal SDRAM_CKE_sig: std_logic;
 signal SDRAM_DQO_sig : std_logic_vector(15 downto 0);
 signal SDRAM_A_sig : std_logic_vector(11 downto 0);
 signal SDRAM_BA_sig : std_logic_vector(1 downto 0);
 signal line2write, line2read, linerw : std_logic_vector(8 downto 0);
 signal rasn_norm : std_logic;
 signal casn_norm : std_logic;
 signal rowaddr_norm : std_logic_vector(11 downto 0);
 signal coladdr_norm : std_logic_vector(11 downto 0);
 signal dec14, dec15, dec16, dec17, dec18 : std_logic;
signal rasn_norm_sig: std_logic;
signal casn_norm_sig: std_logic;
signal rowaddr_norm_sig: std_logic_vector(11 downto 0);
signal coladdr_norm_sig: std_logic_vector(11 downto 0);
signal baddr_norm_sig: std_logic_vector(1 downto 0);
signal count100m_clr, buf_RGB_win_dly1, buf_RGB_win_dly2 : std_logic;
signal cmdready : std_logic;

BEGIN
-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    MemState <= Initial;
  elsif( CLK'event and CLK = '1' ) then
    if(MemState = Initial and pwrupready = '1') then -- go to precharge
      MemState <= PreChAll;
    elsif(MemState = PreChAll) then
      MemState <= doingPreChAll;
    elsif(MemState = doingPreChAll and cmdready = '1') then -- wait 300ns and go to MRS
      MemState <= MRS;
    elsif(MemState = MRS) then
      MemState <= doingMRS;
    elsif(MemState = doingMRS and cmdready = '1') then -- wait 300ns and go to burst write
      MemState <= BstWr;
    elsif(MemState = BstWr and VgaHsync_edge = '1' and oddLine2x = '1') then -- go to burst read
      MemState <= BstRd;
    elsif(MemState = BstRd and VgaHsync_edge = '1' and oddLine2x = '0') then -- go to burst write
      MemState <= BstWr;
    end if;
  end if;
end process;

-------------------------------------------------------------------
  cmdFound <= '1' when MemState = PreChall or MemState = MRS else '0';

-------------------------------------------------------------------
process( RST_N, CLK, cmdFound )
begin
  if( RST_N = '0' or cmdFound = '1' ) then
    cmdcount <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(cmdcount /= 15) then
      cmdcount <= cmdcount + 1; 
    end if;
  end if;
end process;

  cmdready <= '1' when cmdcount = 15 else '0'; -- 20ns * 15 = 300ns
  
-------------------------------------------------------------------
  buf_RGB_win <= '1' when CamPixCount4x(2 downto 0) = "000" else '0'; 

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    buf_RGB_dly3 <= (others => '0');
    buf_RGB_dly2 <= (others => '0');
    buf_RGB_dly1 <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(CamPixCount4x(0) = '0') then
      buf_RGB_dly3 <= buf_RGB_dly2;
      buf_RGB_dly2 <= buf_RGB_dly1;
      buf_RGB_dly1 <= buf_RGB;
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    sdram_data3 <= (others => '0');
    sdram_data2 <= (others => '0');
    sdram_data1 <= (others => '0');
    sdram_data0 <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(buf_RGB_win = '1') then
      sdram_data3 <= buf_RGB_dly3;
      sdram_data2 <= buf_RGB_dly2;
      sdram_data1 <= buf_RGB_dly1;
      sdram_data0 <= buf_RGB;
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    buf_RGB_win_dly2 <= '0';
    buf_RGB_win_dly1 <= '0';
  elsif( CLK100M'event and CLK100M = '1' ) then
    buf_RGB_win_dly2 <= buf_RGB_win_dly1;
    buf_RGB_win_dly1 <= buf_RGB_win;
  end if;
end process;

  count100m_clr <= '1' when buf_RGB_win_dly1 = '1' and buf_RGB_win_dly2 = '0' else '0';

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    count100m <= (others => '0');
  elsif( CLK100M'event and CLK100M = '1' ) then
    if(count100m_clr = '1') then
      count100m <= (others => '0');
    else
      count100m <= count100m + 1;
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    pwrupcount <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(pwrupcount /= 10000) then -- 20ns * 10000 = 200us
      pwrupcount <= pwrupcount + 1;
    end if;
  end if;
end process;

  pwrupready <= '1' when pwrupcount = 10000 else '0';

-------------------------------------------------------------------
  oddLine2x <= '0' when CamPixCount4x < 1567 else '1';

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0') then
    FrameCount <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(CamVsync_edge = '1') then 
      FrameCount <= FrameCount + 1;
    end if;
  end if;
end process;

  frame2write <= FrameCount;
  
  frame2read <= FrameCount-1;
                
  frame <= frame2write when MemState = BstWr else frame2read;
  
-------------------------------------------------------------------
  line2write <= CamLineCount(8 downto 0);
  
  
-------------------------------------------------------------------
  line2read <= VgaLineCount;

-------------------------------------------------------------------
  linerw <= line2write when MemState = BstWr else 
            line2read;
                  
-------------------------------------------------------------------
  rasn_norm_sig <= '0' when dec16 = '1' else '1'; 
  
  casn_norm_sig <= '0' when count100m = 2 else '1';
  
  rowaddr_norm_sig <= frame(0) & linerw & VgaPixCount(9 downto 8);

  coladdr_norm_sig <= "0100" & VgaPixCount(7 downto 2) & "00";
  
  baddr_norm_sig <= frame(2 downto 1);
             
-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    rasn_norm <= '0';
    casn_norm <= '0';
    rowaddr_norm <= (others => '0');
    coladdr_norm <= (others => '0');
    baddr_norm <= (others => '0');
  elsif( CLK100M'event and CLK100M = '1' ) then
    rasn_norm <= rasn_norm_sig;
    casn_norm <= casn_norm_sig;
    rowaddr_norm <= rowaddr_norm_sig; 
    coladdr_norm <= coladdr_norm_sig;
    baddr_norm <= baddr_norm_sig;
  end if;
end process;

-------------------------------------------------------------------
  sdram_data <= sdram_data3 when count100m = 3 else 
                sdram_data2 when count100m = 4 else
                sdram_data1 when count100m = 5 else
                sdram_data0 when count100m = 6 else (others => '0');
                
-------------------------------------------------------------------
  csn <= '0';
  wrn <= '0' when MemState = BstWr and 
                 oddLine2x = '0' and count100m = 3 else '1';
                                                       
  rasn <= rasn_norm;
                         
  casn <= casn_norm; 

  dqm <= '0';
  baddr <= baddr_norm;
  
  dqenbn <= '0' when MemState = BstWr and 
                      oddLine2x = '0' and count100m >= 3 and count100m <= 6 else '1';

  rowaddr <= rowaddr_norm;
                        
  coladdr <= coladdr_norm;
  
-------------------------------------------------------------------
  SDRAM_BA_sig <= "00" when MemState = MRS else baddr;  
  SDRAM_A_sig <= "010000000000" when MemState = PreChAll or MemState = doingPreChAll else
             "000000100010" when MemState = MRS else 
--             "000000000000" when MemState = doingMRS else 
             "000000100010" when MemState = doingMRS else -- same as MRS 2014/04/15
             rowaddr when count100m = 1 or count100m = 9 else 
             coladdr when count100m = 3 or count100m = 11 else (others => '0');
  
  SDRAM_DQO_sig <= sdram_data;  

  SDRAM_DQMH_sig <= '0';
  SDRAM_DQML_sig <= '0';
  
  SDRAM_CS_N_sig <= '0' when MemState = PreChAll or MemState = doingPreChAll else 
                '0' when MemState = MRS else
                '0' when MemState = doingMRS else csn;
                
  SDRAM_WE_N_sig <= '0' when MemState = PreChAll else
                '1' when MemState = doingPreChAll else
                '0' when MemState = MRS else
--                '1' when MemState = doingMRS else wrn;
                '0' when MemState = doingMRS else wrn; -- same as MRS 2014/04/15
                
  SDRAM_RAS_N_sig <= '0' when MemState = PreChAll or MemState = doingPreChAll else
                 '0' when MemState = MRS else
--                 '1' when MemState = doingMRS else rasn;
                 '0' when MemState = doingMRS else rasn; -- same as MRS 2014/04/15
                 
  SDRAM_CAS_N_sig <= '1' when MemState = PreChAll else
                 '0' when MemState = doingPreChAll else
                 '0' when MemState = MRS else
--                 '1' when MemState = doingMRS else  casn;
                 '0' when MemState = doingMRS else  casn; -- same as MRS 2014/04/15
  
  SDRAM_CKE_sig <= '1';

  SDRAM_DQENB_N_sig <= dqenbn;
  
-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0') then
    SDRAM_BA <= (others => '0');
    SDRAM_A <= (others => '0');
    SDRAM_DQO <= (others => '0'); 
    SDRAM_DQMH <= '0';
    SDRAM_DQML <= '0';
    SDRAM_CS_N <= '0';
    SDRAM_WE_N <= '0';
    SDRAM_RAS_N <= '0';
    SDRAM_CAS_N <= '0';
    SDRAM_CKE <= '0';
    SDRAM_DQENB_N <= '0';
  elsif( CLK100M'event and CLK100M = '1' ) then
    SDRAM_BA <= SDRAM_BA_sig;
    SDRAM_A <= SDRAM_A_sig;
    SDRAM_DQO <= SDRAM_DQO_sig;  
    SDRAM_DQMH <= SDRAM_DQMH_sig;
    SDRAM_DQML <= SDRAM_DQML_sig;
    SDRAM_CS_N <= SDRAM_CS_N_sig;
    SDRAM_WE_N <= SDRAM_WE_N_sig;
    SDRAM_RAS_N <= SDRAM_RAS_N_sig;
    SDRAM_CAS_N <= SDRAM_CAS_N_sig;
    SDRAM_CKE <= SDRAM_CKE_sig;
    SDRAM_DQENB_N <= SDRAM_DQENB_N_sig;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    latch0 <= (others => '0');
  elsif( CLK100M'event and CLK100M = '1' ) then
    if(count100m = 6) then 
      latch0 <= SDRAM_DQI;
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    latch1 <= (others => '0');
  elsif( CLK100M'event and CLK100M = '1' ) then
    if(count100m = 7) then 
      latch1 <= SDRAM_DQI;
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    latch2 <= (others => '0');
  elsif( CLK100M'event and CLK100M = '1' ) then
    if(count100m = 8) then 
      latch2 <= SDRAM_DQI;
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    latch3 <= (others => '0');
  elsif( CLK100M'event and CLK100M = '1' ) then
    if(count100m = 9) then 
      latch3 <= SDRAM_DQI;
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    winp0 <= '0';
  elsif( CLK100M'event and CLK100M = '1' ) then
    if((count100m = 6 or count100m = 7) and oddLine2x = '1') then 
      winp0 <= '1';
    else 
      winp0 <= '0';
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    winp1 <= '0';
  elsif( CLK100M'event and CLK100M = '1' ) then
    if((count100m = 8 or count100m = 9) and oddLine2x = '1') then 
      winp1 <= '1';
    else 
      winp1 <= '0';
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    winp2 <= '0';
  elsif( CLK100M'event and CLK100M = '1' ) then
    if((count100m = 10 or count100m = 11) and oddLine2x = '1') then 
      winp2 <= '1';
    else 
      winp2 <= '0';
    end if;
  end if;
end process;

-------------------------------------------------------------------
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    winp3 <= '0';
  elsif( CLK100M'event and CLK100M = '1' ) then
    if((count100m = 12 or count100m = 13) and oddLine2x = '1') then 
      winp3 <= '1';
    else 
      winp3 <= '0';
    end if;
  end if;
end process;

-------------------------------------------------------------------
  dec14 <= '1' when count100m = 14 else '0';
  
process( RST_N, CLK100M )
begin
  if( RST_N = '0' ) then
    dec18 <= '0';
    dec17 <= '0';
    dec16 <= '0';
    dec15 <= '0';
  elsif( CLK100M'event and CLK100M = '1' ) then
    dec18 <= dec17;
    dec17 <= dec16;
    dec16 <= dec15;
    dec15 <= dec14;
  end if;
end process;

END structure;
