-- Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VideoProc is
	PORT	(
	CLK	: IN std_logic;
	RST_N	: IN std_logic;
	XCLK : out std_logic;
  CARD_CMD : out std_logic;
  CARD_CLK : out std_logic;
  CARD_DAT0 : in std_logic;
  CARD_DAT3 : out std_logic;
	CamHsync	: IN std_logic;
	CamVsync	: IN std_logic;
	PCLK	: IN std_logic;
  CamData	: in std_logic_vector(7 downto 0);
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQ : inout std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK : out std_logic;
	SCL : inout std_logic;
	SDA : inout std_logic;
		data0p	: out std_logic_vector(0 downto 0);
		data0n	: out std_logic_vector(0 downto 0);
		data1p	: out std_logic_vector(0 downto 0);
		data1n	: out std_logic_vector(0 downto 0);
		data2p	: out std_logic_vector(0 downto 0);
		data2n	: out std_logic_vector(0 downto 0);
		clockp	: out std_logic_vector(0 downto 0);
		clockn	: out std_logic_vector(0 downto 0)	);
end VideoProc ;

architecture structure of VideoProc is

component VideoProcCore
	PORT	(
	CLK	: IN std_logic;
	CLK100M	: IN std_logic;
	RST_N	: IN std_logic;
	XCLK : out std_logic;
	CamHsync	: IN std_logic;
	CamVsync	: IN std_logic;
	PCLK	: IN std_logic;
  CamData	: in std_logic_vector(7 downto 0);
	SDRAM_DQENB_N : out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK : out std_logic;
 		DCLK : in std_logic;
		SCLK : in std_logic;

		data0p_h	: out std_logic;
		data0p_l	: out std_logic;
		data0n_h	: out std_logic;
		data0n_l	: out std_logic;

		data1p_h	: out std_logic;
		data1p_l	: out std_logic;
		data1n_h	: out std_logic;
		data1n_l	: out std_logic;

		data2p_h	: out std_logic;
		data2p_l	: out std_logic;
		data2n_h	: out std_logic;
		data2n_l	: out std_logic;

		clockp_h	: out std_logic;
		clockp_l	: out std_logic;
		clockn_h	: out std_logic;
		clockn_l	: out std_logic );
end component;

	component dviddio
	port
	(
		outclock : in  std_logic                    := '0';             -- outclock.export
		din      : in  std_logic_vector(1 downto 0) := (others => '0'); --      din.export
		pad_out  : out std_logic_vector(0 downto 0)                     --  pad_out.export
	);
	end component;

	component dvipll
	PORT
	(
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC;			-- 50MHz
		c0			: OUT STD_LOGIC;		-- 25MHz
		c1			: OUT STD_LOGIC;		-- 125MHz
		locked		: OUT STD_LOGIC 
	);
	end component;

    component nios2e is
        port (
            clk_clk                        : in    std_logic := 'X'; -- clk
            reset_reset_n                  : in    std_logic := 'X'; -- reset_n
            scl_external_connection_export : inout std_logic := 'X'; -- export
            sda_external_connection_export : inout std_logic := 'X'  -- export
        );
    end component nios2e;

component sdrampll 
	PORT
	(
		areset		: IN STD_LOGIC;
		inclk0		: IN STD_LOGIC;
		c0		: OUT STD_LOGIC ;
		locked		: OUT STD_LOGIC 
	);
end component;

signal DCLK, SCLK : std_logic;
signal data0p_h, data0p_l, data0n_h, data0n_l : std_logic;	
signal data1p_h, data1p_l, data1n_h, data1n_l : std_logic;
signal data2p_h, data2p_l, data2n_h, data2n_l : std_logic;
signal clockp_h, clockp_l, clockn_h, clockn_l : std_logic;
signal CLK100M : std_logic;
 signal SDRAM_DQENB_N : std_logic;
signal   SDRAM_DQI, SDRAM_DQO : std_logic_vector(15 downto 0);

begin

    u0 : component nios2e
        port map (
            clk_clk                        => CLK,                        
            reset_reset_n                  => RST_N,                  
            scl_external_connection_export => SCL, 
            sda_external_connection_export => SDA  
     );

VideoProcCore_1 : VideoProcCore port map (
		CLK	=> CLK,
		CLK100M	=> CLK100M,
		RST_N	=> RST_N,
		XCLK	=> XCLK,
	CamHsync	=> CamHsync,
	CamVsync	=> CamVsync,
	PCLK	=> PCLK,
  CamData	=> CamData,
     SDRAM_DQENB_N => SDRAM_DQENB_N,
  SDRAM_DQMH	=> SDRAM_DQMH,
  SDRAM_DQML	=> SDRAM_DQML,
  SDRAM_CS_N	=> SDRAM_CS_N,
  SDRAM_WE_N	=> SDRAM_WE_N,
  SDRAM_RAS_N	=> SDRAM_RAS_N,
  SDRAM_CAS_N	=> SDRAM_CAS_N,
  SDRAM_CKE	=> SDRAM_CKE,
  SDRAM_DQI => SDRAM_DQI,
  SDRAM_DQO => SDRAM_DQO,
  SDRAM_A => SDRAM_A,
  SDRAM_BA => SDRAM_BA,
  SDRAM_CLK	=> SDRAM_CLK,
 		DCLK => DCLK,
		SCLK => SCLK,

	data0p_h	=> data0p_h,
	data0p_l	=> data0p_l,
	data0n_h	=> data0n_h,
	data0n_l	=> data0n_l,

	data1p_h	=> data1p_h,
	data1p_l	=> data1p_l,
	data1n_h	=> data1n_h,
	data1n_l	=> data1n_l,

	data2p_h	=> data2p_h,
	data2p_l	=> data2p_l,
	data2n_h	=> data2n_h,
	data2n_l	=> data2n_l,

	clockp_h	=> clockp_h,
	clockp_l	=> clockp_l,
	clockn_h	=> clockn_h,
	clockn_l	=> clockn_l );
		
	TX0_P : dviddio
		port map (
			outclock	=> SCLK,
			din	=> data0p_l & data0p_h,
			pad_out		=> data0p
		);
		
	TX0_N : dviddio
		port map (
			outclock	=> SCLK,
			din	=> data0n_l & data0n_h,
			pad_out		=> data0n
		);
		
	TX1_P : dviddio
		port map (
			outclock	=> SCLK,
			din	=> data1p_l & data1p_h,
			pad_out		=> data1p
		);
		
	TX1_N : dviddio
		port map (
			outclock	=> SCLK,
			din	=> data1n_l & data1n_h,
			pad_out		=> data1n
		);
		
	TX2_P : dviddio
		port map (
			outclock	=> SCLK,
			din	=> data2p_l & data2p_h,
			pad_out		=> data2p
		);
		
	TX2_N : dviddio
		port map (
			outclock	=> SCLK,
			din	=> data2n_l & data2n_h,
			pad_out		=> data2n
		);
		
	TXC_P : dviddio
		port map (
			outclock	=> SCLK,
			din	=> clockp_l & clockp_h,
			pad_out		=> clockp
		);
		
	TXC_N : dviddio
		port map (
			outclock	=> SCLK,
			din	=> clockn_l & clockn_h,
			pad_out		=> clockn
		);
		
	dvipll_1 : dvipll
		port map (
			areset	=> '0',
			inclk0	=> CLK,
			c0		=> DCLK,
			c1		=> SCLK,
			locked	=> open
		);
		
-------------------------------------------------------------- 
  sdrampll_1 : sdrampll 
    port map(
		areset	=> '0',
		inclk0	=> CLK,
		c0		=> CLK100M,
		locked	=> open );

-------------------------------------------------------------- 
  SDRAM_DQI <= SDRAM_DQ;
  SDRAM_DQ <= SDRAM_DQO when SDRAM_DQENB_N = '0' else (others => 'Z');
  
end structure ;
