// Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

module VideoProc (
	CLK,
	RST_N,
	XCLK,
	SCL,
	SDA,
	CamHsync,
	CamVsync,
	PCLK,
  CamData,
  SDRAM_DQMH,
  SDRAM_DQML,
  SDRAM_CS_N,
  SDRAM_WE_N,
  SDRAM_RAS_N,
  SDRAM_CAS_N,
  SDRAM_CKE,
  SDRAM_DQ,
  SDRAM_A,
  SDRAM_BA,
  SDRAM_CLK,
		data0p,
		data0n,
		data1p,
		data1n,
		data2p,
		data2n,
		clockp,
		clockn,
); 

input		CLK, RST_N;
output	XCLK;
inout SCL, SDA;
input	CamHsync;
input CamVsync;
input PCLK;
input [7:0]CamData;
output [0:0] data0p, data0n, data1p, data1n, data2p, data2n, clockp,	clockn;
output SDRAM_DQMH;
output SDRAM_DQML;
output SDRAM_CS_N;
output SDRAM_WE_N;
output SDRAM_RAS_N;
output SDRAM_CAS_N;
output SDRAM_CKE;
inout[15:0] SDRAM_DQ;
output[11:0] SDRAM_A;
output[1:0] SDRAM_BA;
output SDRAM_CLK;

wire DCLK, SCLK;
wire data0p_h, data0p_l, data0n_h, data0n_l;
wire data1p_h, data1p_l, data1n_h, data1n_l;
wire data2p_h, data2p_l, data2n_h, data2n_l;
wire clockp_h, clockp_l, clockn_h, clockn_l;
wire CLK100M;
wire SDRAM_DQENB_N;
wire [15:0] SDRAM_DQI, SDRAM_DQO;

	VideoProcCore VideoProcCore_inst(
		.CLK(CLK),
		.CLK100M(CLK100M),
		.RST_N(RST_N),
	.XCLK(XCLK),
	.CamHsync(CamHsync),
	.CamVsync(CamVsync),
	.PCLK(PCLK),
  .CamData(CamData),
  .SDRAM_DQENB_N(SDRAM_DQENB_N),
  .SDRAM_DQMH(SDRAM_DQMH),
  .SDRAM_DQML(SDRAM_DQML),
  .SDRAM_CS_N(SDRAM_CS_N),
  .SDRAM_WE_N(SDRAM_WE_N),
  .SDRAM_RAS_N(SDRAM_RAS_N),
  .SDRAM_CAS_N(SDRAM_CAS_N),
  .SDRAM_CKE(SDRAM_CKE),
  .SDRAM_DQI(SDRAM_DQI),
  .SDRAM_DQO(SDRAM_DQO),
  .SDRAM_A(SDRAM_A),
  .SDRAM_BA(SDRAM_BA),
  .SDRAM_CLK(SDRAM_CLK),

		.DCLK(DCLK),
		.SCLK(SCLK),

		.data0p_h(data0p_h),
		.data0p_l(data0p_l),
		.data0n_h(data0n_h),
		.data0n_l(data0n_l),

		.data1p_h(data1p_h),
		.data1p_l(data1p_l),
		.data1n_h(data1n_h),
		.data1n_l(data1n_l),

		.data2p_h(data2p_h),
		.data2p_l(data2p_l),
		.data2n_h(data2n_h),
		.data2n_l(data2n_l),

		.clockp_h(clockp_h),
		.clockp_l(clockp_l),
		.clockn_h(clockn_h),
		.clockn_l(clockn_l)
	);

	dviddio TX0_P(
		.outclock(SCLK),
		.din({data0p_l, data0p_h}),
		.pad_out(data0p)
	);
	
	dviddio TX0_N(
		.outclock(SCLK),
		.din({data0n_l, data0n_h}),
		.pad_out(data0n)
	);
	
	dviddio TX1_P(
		.outclock(SCLK),
		.din({data1p_l, data1p_h}),
		.pad_out(data1p)
	);
	
	dviddio TX1_N(
		.outclock(SCLK),
		.din({data1n_l, data1n_h}),
		.pad_out(data1n)
	);
	
	dviddio TX2_P(
		.outclock(SCLK),
		.din({data2p_l, data2p_h}),
		.pad_out(data2p)
	);
	
	dviddio TX2_N(
		.outclock(SCLK),
		.din({data2n_l, data2n_h}),
		.pad_out(data2n)
	);
	
	dviddio TXC_P(
		.outclock(SCLK),
		.din({clockp_l, clockp_h}),
		.pad_out(clockp)
	);
	
	dviddio TXC_N(
		.outclock(SCLK),
		.din({clockn_l, clockn_h}),
		.pad_out(clockn)
	);
	
	dvipll dvipll_1(
		.areset(1'b0),
		.inclk0(CLK),
		.c0(DCLK),
		.c1(SCLK),
		.locked()
	);	

	 nios2e u0 (
        .clk_clk                        (CLK),                        
        .reset_reset_n                  (RST_N),                  
        .scl_external_connection_export (SCL), 
        .sda_external_connection_export (SDA)  
    );
	 
  	 sdrampll sdrampll_inst (
        .areset (1'b0),
        .inclk0 (CLK),
        .c0 (CLK100M),
        .locked () 
    );
	 
  assign SDRAM_DQI = SDRAM_DQ;
  assign SDRAM_DQ = SDRAM_DQENB_N == 0 ? SDRAM_DQO : 16'bz;

endmodule
